import numpy as np
from planer import resize
import planer as rt

root = '/'.join(__file__.split('\\')[:-1])+'/models'

def load():
    with open(root+'/names.txt') as f: globals()['labs'] = f.read().split('\n')
    globals()['net'] = rt.InferenceSession(root+'/yolov4.onnx')

def preprocess(img, size):
    (h, w), (H, W) = img.shape[:2], size
    k = min(H/h, W/w)
    nh, nw = int(h*k+0.5)//2*2, int(w*k+0.5)//2*2
    img.shape = img.shape[:2] + (-1,)
    img = resize(img, (nh, nw), backend=np)
    k = np.ones((1,1,3), dtype='uint8')
    img = img / (k * 255)
    dh, dw = (H-nh)//2, (W-nw)//2
    pads = [(dh,dh),(dw,dw),(0,0)]
    return np.pad(img, pads, constant_values=0.5)

def iou(box1, box2):
    (xa1, ya1, xa2, ya2), (xb1, yb1, xb2, yb2) = box1, box2
    area1 = (xa2 - xa1) * (ya2 - ya1)
    area2 = (xb2 - xb1) * (yb2 - yb1)
    ix = max(min(xa2, xb2) - max(xa1, xb1), 0)
    iy = max(min(ya2, yb2) - max(ya1, yb1), 0)
    return (ix * iy) / (area1 + area2 - ix * iy)

def nms_filter(boxes, iou_threshold=0.45):
    boxes = boxes[np.argsort(-boxes[:,5] * 10000 - boxes[:,4])]
    sep = (np.where(boxes[1:,5] != boxes[:-1,5])[0] + 1).tolist()
    for s, e in zip([None]+sep, sep+[None]):
        boxcls = boxes[s:e]
        for i in range(len(boxcls)):
            if boxcls[i,4]==0: continue
            for j in range(i+1, len(boxcls)):
                if boxcls[j,4]==0: continue
                v = iou(boxcls[i,:4], boxcls[j,:4])
                if v > iou_threshold: boxcls[j,4]=0
    return boxes[boxes[:,4]>0]

def combine_boxes(xyfs, anchors, strides, scales=[1,1,1]):
    for i, pred in enumerate(xyfs):
        xy_grid = np.mgrid[:pred.shape[1], :pred.shape[1]]
        xy_grid = xy_grid.T[:,:,None]
        pred_xy = np.exp(-pred[:, :, :, :, 0:2])
        pred_xy = scales[i]/(1 + pred_xy)
        pred_xy -= 0.5 * (scales[i] - 1) - xy_grid
        pred_wh = np.exp(pred[:, :, :, :, 2:4])
        pred_xy *= strides[i]; pred_wh *= anchors[i]
        pred[..., 0:2], pred[..., 2:4] = pred_xy, pred_wh
    boxes = [i.reshape(-1, i.shape[-1]) for i in xyfs]
    return np.concatenate(boxes, axis=0)

def good_boxes(xyfs, conf_thres=0.25, max_nms=3000):
    xyfs = xyfs[xyfs[:,4]>conf_thres]
    xyf = np.zeros((len(xyfs), 6), dtype=np.float32)
    xyf[:, [0,1]] = xyfs[:, [0,1]] - xyfs[:, [2,3]]/2
    xyf[:, [2,3]] = xyfs[:, [0,1]] + xyfs[:, [2,3]]/2
    xyf[:, 4] = xyfs[:, 4] * xyfs[:, 5:].max(axis=1)
    xyf[:, 5] = xyfs[:, 5:].argmax(axis=1)
    xyf = xyf[xyf[:, 4] > conf_thres]
    if not xyf.shape[0]: return None
    if len(xyf) > max_nms:  # excess boxes
        xyf = xyf[xyf[:, 4].argsort(True)[:max_nms]]
    return xyf
        
def scale_boxes(coords, des, ori):
    gain = min(des[0] / ori[0], des[1] / ori[1])
    pad = (des[1] - ori[1] * gain) / 2, (des[0] - ori[0] * gain) / 2
    coords[:, :4] -= pad[0], pad[1], pad[0], pad[1]
    coords[:, [0,2]] = np.clip(coords[:, [0,2]]/gain, 0, ori[1]-1)
    coords[:, [1,3]] = np.clip(coords[:, [1,3]]/gain, 0, ori[0]-1)
    return coords

def get_boxes(xyfs, shp, conf_thr=0.25, iou_thr=0.45, size=(416, 416)):
    boxes = good_boxes(xyfs, conf_thr)
    boxes = nms_filter(boxes, iou_thr)
    return scale_boxes(boxes, size, shp)

anchors = np.array([
    [[ 12, 16],[ 19, 36],[ 40,  28]],
    [[ 36, 75],[ 76, 55],[ 72, 146]],
    [[142,110],[192,243],[459, 401]]], dtype=np.float32)

def detect(img, conf_thr=0.25, iou_thr=0.45):
    x = preprocess(img, (416, 416))[None,:]
    y = net(rt.asarray(x))
    y = [rt.asnumpy(i) for i in y]
    y = combine_boxes(y, anchors, [8, 16, 32], [1.2, 1.1, 1.05])
    boxes = get_boxes(y, img.shape, conf_thr, iou_thr, (416, 416))
    rst = []
    for x1,y1,x2,y2,p,c in boxes:
        rst.append((int(x1), int(y1), int(x2), int(y2), p, labs[int(c)]))
    return rst

def show(image, boxes):
    import matplotlib.pyplot as plt
    plt.imshow(image)
    for x1,y1,x2,y2,p,c in boxes:
        plt.plot([x1,x2,x2,x1,x1], [y1,y1,y2,y2,y1], 'blue')
        plt.text(x1,y1,'%s:%.2f'%(c,p),
            color='white', verticalalignment='top')
    plt.show()

def test():
    from imageio import imread
    img = imread(root+'/bus.jpg')
    boxes = detect(img)    
    show(img, boxes)
    
if __name__ == '__main__':
    load()
    test()



import planer
from planer import tile
import numpy as np


root = '/'.join(__file__.split('\\')[:-1])+'/models'

def load():
    globals()['A'] = planer.read_net(root+'/A_enc.onnx')
    globals()['B'] = planer.read_net(root+'/B_dec.onnx')
    globals()['mapping'] = planer.read_net(root+'/mapping.onnx')

@tile(glob=128, window=256)
def enhance(img):
    x = img.astype('float32')
    x = x.transpose(2,0,1)[None,:]
    x -= 0.5; x *= 2
    feat = A.run(None, {'input':x})[0]
    feat_map = mapping.run(None, {'input':feat})[0]
    y = B.run(None, {'input':feat_map})[0]
    y += 1; y /= 2
    return y[0].transpose(1,2,0)

def test():
    from matplotlib import pyplot as plt
    from imageio import imread
    
    img = imread(root+'/old_photo.png')
    y = enhance(img, sample=1, window=1024)

    plt.subplot(121)
    plt.imshow(img)

    plt.subplot(122)
    plt.imshow(y)
    plt.show()
    
if __name__ == '__main__':
    load()
    test()



import planer
from planer import tile
# import onnxruntime as rt

from scipy.ndimage import distance_transform_edt as edt

root = '/'.join(__file__.split('\\')[:-1])+'/models'
def load(): 
    globals()['net'] = planer.read_net(root+'/fba.onnx')

def prework(img, mark):
    import numpy as np
    img = img.astype(np.float32)/255
    mark = mark.astype(np.float32)/255
    
    trimap = np.array([mark==0, mark==1])
    trimap = trimap.astype(np.float32)

    offset = np.array([0.485, 0.456, 0.406])
    nimg = img - offset.astype('float32')[None,None,:]
    nimg /= np.array([0.229, 0.224, 0.225])

    L, w = 320, [0.02, 0.08, 0.16]
    k = 2 * (np.array(w)*L)[:,None,None]**2
    dis0 = edt(trimap[0]!=1); dis0 **= 2
    dis1 = edt(trimap[1]!=1); dis1 **= 2
    y = np.vstack((dis0[None,:]/k, dis1[None,:]/k))
    ntrimap = np.exp(-y, out=y).astype(np.float32)

    return np.concatenate([
        img, trimap.transpose(1,2,0),
        nimg, ntrimap.transpose(1,2,0)], axis=-1)
    
@tile(glob=48)
def matting(img_mark):
    img_mark = img_mark.transpose(2,0,1)[None,:]
    img = img_mark[:,0:3]
    trimap = img_mark[:,3:5]
    nimg = img_mark[:,5:8]
    ntrimap = img_mark[:,8:14]
    para = {'img':img, '1':trimap, '2':nimg, '3':ntrimap}
    y = net.run(None, para)[0]
    return y[0].transpose(1,2,0)

def test():
    from imageio import imread
    import matplotlib.pyplot as plt

    img = imread(root+'/image.png')
    mark = imread(root+'/trimap.png')
    img_mark = prework(img, mark)
    y = matting(img_mark)

    plt.subplot(221).imshow(img)
    plt.subplot(222).imshow(y[:,:,0])
    plt.subplot(223).imshow(y[:,:,1:4])
    plt.subplot(224).imshow(y[:,:,4:7])
    plt.show()
    
if __name__ == '__main__':
    load()
    test()



import planer, json
import numpy as np

root = '/'.join(__file__.split('\\')[:-1])+'/models'

def load(name='resnet'):
    globals()['net'] = planer.read_net(root+'/%s.onnx'%name)
    with open(root+'/imagenet_labs.json') as f:
        globals()['classes'] = json.load(f)

def recognize(img):
    img = planer.asarray(img)
    x = planer.resize(img, (224, 224))
    x = x[None, :,:,:].astype('float32')
    x = (x/255).transpose(0, 3, 1, 2)
    y = np.argmax(net(x), axis=-1)
    return classes[str(y[0])]

def test():
    import matplotlib.pyplot as plt
    from imageio import imread
    img = imread(root+'/bus.jpg')
    img = np.asnumpy(img)
    rst = recognize(img)
    plt.imshow(img)
    plt.title(rst)
    plt.show()
    
if __name__ == '__main__':
    load('mobilenet')
    test()



import planer as rt
import numpy as np

root = '/'.join(__file__.split('\\')[:-1])+'/models'
def load(): 
    globals()['net'] = rt.InferenceSession(root+'/real_esrgan_x4.onnx')

@rt.tile(window=512)
def enhance(img):
    img = img.astype(np.float32)/255
    img = img.transpose(2,0,1)[None,:]
    y = net.run(None, {'input.1':img})[0]
    y = y[0].transpose(1,2,0)
    y = np.clip(y, 0, 1, out = y)
    return (y * 255).astype(np.uint8)
    
def test():
    from imageio import imread
    import matplotlib.pyplot as plt

    img = imread(root + '/mushroom.jpg')[::3,::3]
    y = enhance(img)

    plt.subplot(121).imshow(img)
    plt.subplot(122).imshow(y)
    plt.show()

if __name__ == '__main__':
    test()



import planer, os
import numpy as np

root = '/'.join(__file__.split('\\')[:-1])+'/models'

def load():
    globals()['net'] = planer.read_net(root+'/face68key.onnx')
    
# get pixels on the line
def line(r1, c1, r2, c2):
    d = max(abs(r1-r2), abs(c1-c2)) + 1
    rs = np.linspace(r1,r2,d).round()
    cs = np.linspace(c1,c2,d).round()
    return rs.astype(int), cs.astype(int)

# count flow from kay points
# weights: ear - face - jaw 's weights, 0 means stay there.
# fac: + means fat, - means thin, 0 would got a zeros flow
def count_flow(rc, weights=[0,1,2,2,2,2,1,1,0], fac=5):
    np = planer.np
    flow = np.zeros((224,224,2), dtype=np.float32)
    l = rc[:17].astype(np.int)
    nose = rc[31:36].mean(axis=0)
    dv = nose - l
    dv.T[:] *= weights + weights[1:]
    for s,e,v1,v2 in zip(l[:-1], l[1:], dv[:-1], dv[1:]):
        flow[line(*s, *e)] = (v1+v2)/2
    flow = planer.gaussian_filter(flow, 20)
    nv = np.linalg.norm(flow, axis=-1)
    return flow * (fac/nv.max())

# apply the flow
def flow_map(img, flow):
    np = planer.np
    des = np.mgrid[0:224, 0:224].transpose(1,2,0) + flow
    des = planer.resize(des, img.shape[:2])
    des *= np.array(img.shape[:2]).reshape(1,1,2)/224
    return planer.mapcoord(img, *des.transpose(2,0,1))

# generate check board
def check_board(shp, d=20):
    checkboard = np.zeros(shp, dtype=np.uint8)
    msk = np.tile([False]*d+[True]*d, 1000)
    checkboard[msk[:shp[0]]] += 128
    checkboard[:,msk[:shp[1]]] += 128
    return ((checkboard>0)*255).astype(np.uint8)

def get_face_key(img, scale=True):
    img = planer.asarray(img, dtype='float32')
    x = planer.resize(img, (224,224))/255
    if img.ndim==2: x = x[:,:,None]
    x = x.transpose(2,0,1)[None,:,:,:]
    rc = net(x).reshape(-1,2)[:,::-1] * 50 + 100
    rc = planer.asnumpy(rc)
    if scale: rc *= np.array(img.shape[:2])/224
    return rc

def face_adjust(img, weights=[0,1,2,2,2,2,1,1,0], fac=5):
    img = planer.asarray(img)
    rc = get_face_key(img, False)
    flow = count_flow(rc, weights, fac)
    return planer.asnumpy(flow_map(img, flow))

def test():
    import matplotlib.pyplot as plt
    from imageio import imread
    face = imread(root+'/benshan.jpg')

    rc = get_face_key(face)
    plt.subplot(131).imshow(face)
    plt.plot(*rc.T[::-1], 'r.')
    plt.title('68 key points')
    
    thin = face_adjust(face, fac=-10)
    plt.subplot(133).imshow(thin)
    plt.title('thin with fac -10')
    
    fat = face_adjust(face, fac=10)
    plt.subplot(132).imshow(fat)
    plt.title('fat with fac 10')
    plt.show()
    
if __name__ == '__main__':
    load()
    test()
    



import numpy as np
import planer

root = '/'.join(__file__.split('\\')[:-1])+'/models'
def load(): 
    globals()['net'] = planer.read_net(root+'/u2net.onnx')
    
def global_norm(img):
    img.shape = img.shape[:2]+(-1,)
    img = img.astype(np.float32)
    img -= img.min(); img /= img.max()
    offset = [0.485, 0.456, 0.406]
    offset = np.array(offset, dtype=np.float32)
    img = img - offset[None,None,:]
    img /= np.array([0.229, 0.224, 0.225])
    return img

@planer.tile(glob=32)
def matting(img):
    img = img.transpose(2,0,1)[None,:,:,:]
    d1,d2,d3,d4,d5,d6,d7 = net(img)
    rst = d1[0,0,:,:]
    rst -= rst.min()
    rst /= rst.max()
    return rst

def test():
    from imageio import imread
    import matplotlib.pyplot as plt
    img = imread(root+'/astronaut.jpg')
    normimg = global_norm(img)
    rst = matting(normimg, sample=1, window=512)
    plt.subplot(121).imshow(img)
    plt.subplot(122).imshow(1-rst, cmap='gray')
    plt.show()
    
if __name__ == '__main__':
    load()
    test()



import planer
import numpy as np
import scipy.ndimage as ndimg
from time import time

root = '/'.join(__file__.split('\\')[:-1])+'/models'

def load(name='cyto_v1'):
    globals()['net'] = planer.read_net(root+'/%s.onnx'%name)

@planer.tile(glob=64)
def count_flow(img):
    img = img[None, :, :]/255
    img = np.concatenate((img, img*0))
    y, style = net(img[None,:,:,:])
    y[0,2] = 1/(1+np.e**-y[0,2])
    return y[0].transpose(1,2,0)

def estimate_volumes(arr, sigma=3):
    msk = arr > 50; 
    idx = np.arange(len(arr), dtype=np.uint32)
    idx, arr = idx[msk], arr[msk]
    for k in np.linspace(5, sigma, 5):
       std = arr.std()
       dif = np.abs(arr - arr.mean())
       msk = dif < std * k
       idx, arr = idx[msk], arr[msk]
    return arr.mean(), arr.std()

def flow2msk(flowp, level=0.5, grad=0.5, area=None, volume=None):
    shp, dim = flowp.shape[:-1], flowp.ndim - 1
    l = np.linalg.norm(flowp[:,:,:2], axis=-1)
    flow = flowp[:,:,:2]/l.reshape(shp+(1,))
    flow[(flowp[:,:,2]<level)|(l<grad)] = 0
    ss = ((slice(None),) * (dim) + ([0,-1],)) * 2
    for i in range(dim):flow[ss[dim-i:-i-2]+(i,)]=0
    sn = np.sign(flow); sn *= 0.5; flow += sn;
    dn = flow.astype(np.int32).reshape(-1, dim)
    strides = np.cumprod(np.array((1,)+shp[::-1]))
    dn = (strides[-2::-1] * dn).sum(axis=-1)
    rst = np.arange(flow.size//dim); rst += dn
    for i in range(10): rst = rst[rst]
    hist = np.bincount(rst, None, len(rst))
    hist = hist.astype(np.uint32).reshape(shp)
    lab, n = ndimg.label(hist, np.ones((3,)*dim))
    volumes = ndimg.sum(hist, lab, np.arange(n+1))
    areas = np.bincount(lab.ravel())
    mean, std = estimate_volumes(volumes, 2)
    if not volume: volume = max(mean-std*3, 50)
    if not area: area = volumes // 3
    msk = (areas<area) & (volumes>volume)
    lut = np.zeros(n+1, np.uint32)
    lut[msk] = np.arange(1, msk.sum()+1)
    return lut[lab].ravel()[rst].reshape(shp)

def flow2hsv(flow):
    v = np.linalg.norm(flow, axis=-1)
    h = np.arccos(flow[:,:,0]/v)
    h *= np.sign(flow[:,:,1])/(np.pi*2)
    h += 0.5; v /= v.max()
    s = np.ones_like(v)             
    a = np.floor(h * 6)
    b = h * 6; b -= a
    p = np.zeros_like(v)
    t = v * b; q = v - t
    buf = np.stack((v,t,p,q), -1).ravel()
    buf *= 255; buf = buf.astype(np.uint8)
    idx = np.array([[0,1,3],[3,0,2],
        [2,0,1], [2,3,0],[1,2,0],[0,2,3]])
    idx = idx[a.ravel().astype(np.uint8) % 6]
    idx += np.arange(v.size)[:,None] * 4
    return buf[idx].reshape(v.shape+(3,))

def draw_edge(img, lab, color=(255,0,0)):
    msk = np.zeros(lab.shape, dtype=np.bool)
    mskr = lab[1:] != lab[:-1]
    mskc = lab[:,1:] != lab[:,:-1]
    msk[1:] |= mskr; msk[:-1] |= mskr
    msk[:,1:] |= mskc; msk[:,:-1] |= mskc
    lut = np.array([[0,0,0],color], dtype=np.uint8)
    rgb = lut[msk.view(np.uint8)]
    img = img.reshape((img.shape+(1,))[:3])
    return np.maximum(img, rgb, out=rgb)

def rgb_mask(img, lab):
    cmap = np.array([(0,0,0), (255,0,0),
        (0,255,0), (0,0,255), (255,255,0),
        (255,0,255), (0,255,255)], np.uint8)
    msk = lab > 0; lab %= 6; lab += 1
    rgb = cmap[lab * msk]
    img = img.reshape((img.shape+(1,))[:3])
    return np.maximum(img, rgb, out=rgb)

def test():
    import matplotlib.pyplot as plt
    from imageio import imread
    img = imread(root + '/cell.png')[:,:,0]

    flow = count_flow(img, sample=1, window=512, margin=0.1)
    lab = flow2msk(flow, level=0.2)
    edge = draw_edge(img, lab)
    rgb = rgb_mask(img, lab)

    plt.subplot(221).imshow(img)
    plt.subplot(222).imshow(flow2hsv(flow))
    plt.subplot(223).imshow(edge)
    plt.subplot(224).imshow(rgb)
    plt.show()
    
if __name__ == '__main__':
    load()
    test()
    



from setuptools import setup, find_packages
import os

descr = """Some useful models based on planer"""

def get_data_files():
    dic = {}
    for root, dirs, files in os.walk('planer_zoo', True):
        root = root.replace('/', '.').replace('\\', '.')
        files = [i for i in files if '.md' in i]
        if len(files)==0:continue
        dic[root] = files
    return dic

if __name__ == '__main__':
    setup(name='planer-zoo',
        version='0.14',
        url='https://github.com/Image-Py/planer-zoo',
        description='toolbox of planer',
        long_description=descr,
        author='YXDragon',
        author_email='yxdragon@imagepy.org',
        license='BSD 3-clause',
        packages=find_packages(),
        package_data=get_data_files(),
        install_requires=[
            'planer',
            'tqdm'
        ],
    )



import importlib, planer

def load_model(name, auto=True):
    model = importlib.import_module('.'+name, 'planer_zoo')
    return planer.Model(model, auto)

if __name__ == '__main__':
    a = list_source('./yolo/v5/yolo5.md')
